@@ -48,8 +48,8 @@
  *
  */
 
-static long
-conv_F_8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_F_8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -71,11 +71,10 @@ conv_F_8 (unsigned char *src, unsigned char *dst, long samples)
       dst += 1;
       src += 4;
     }
-  return samples;
 }
 
-static long
-conv_F_16 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_F_16 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -97,13 +96,12 @@ conv_F_16 (unsigned char *src, unsigned char *dst, long samples)
       dst += 2;
       src += 4;
     }
-  return samples;
 }
 
 
 
-static long
-conv_8_F (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_8_F (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -113,11 +111,10 @@ conv_8_F (unsigned char *src, unsigned char *dst, long samples)
       dst             += 4;
       src             += 1;
     }
-  return samples;
 }
 
-static long
-conv_16_F (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_16_F (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -127,11 +124,10 @@ conv_16_F (unsigned char *src, unsigned char *dst, long samples)
       dst             += 4;
       src             += 2;
     }
-  return samples;
 }
 
-static long
-conv_F_D (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_F_D (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -141,11 +137,10 @@ conv_F_D (unsigned char *src, unsigned char *dst, long samples)
       dst            += 8;
       src            += 4;
     }
-  return samples;
 }
 
-static long
-conv_D_F (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_D_F (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -155,10 +150,9 @@ conv_D_F (unsigned char *src, unsigned char *dst, long samples)
       dst           += 4;
       src           += 8;
     }
-  return samples;
 }
-static long
-conv_16_8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_16_8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -169,11 +163,10 @@ conv_16_8 (unsigned char *src, unsigned char *dst, long samples)
       dst                     += 1;
       src                     += 2;
     }
-  return samples;
 }
 
-static long
-conv_8_16 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_8_16 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -183,172 +176,151 @@ conv_8_16 (unsigned char *src, unsigned char *dst, long samples)
       dst                      += 2;
       src                      += 1;
     }
-  return samples;
 }
 
 /*********/
-static long
-conv_rgbaF_rgba8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbaF_rgba8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_F_8 (src, dst, samples * 4);
-  return samples;
+  conv_F_8 (conversion, src, dst, samples * 4);
 }
 
 #define conv_rgbaF_rgbP8    conv_rgbaF_rgba8
 
-static long
-conv_rgbF_rgb8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbF_rgb8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_F_8 (src, dst, samples * 3);
-  return samples;
+  conv_F_8 (conversion, src, dst, samples * 3);
 }
 
-static long
-conv_gaF_ga8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_gaF_ga8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_F_8 (src, dst, samples * 2);
-  return samples;
+  conv_F_8 (conversion, src, dst, samples * 2);
 }
 
 #define conv_rgbAF_rgbA8    conv_rgbaF_rgba8
 #define conv_gF_g8          conv_F_8
 #define conv_gAF_gA8        conv_gaF_ga8
 
-static long
-conv_rgbaF_rgba16 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbaF_rgba16 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_F_16 (src, dst, samples * 4);
-  return samples;
+  conv_F_16 (conversion, src, dst, samples * 4);
 }
 
-static long
-conv_rgbaF_rgbaD (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbaF_rgbaD (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_F_D (src, dst, samples * 4);
-  return samples;
+  conv_F_D (conversion, src, dst, samples * 4);
 }
 
-static long
-conv_rgbaD_rgbaF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbaD_rgbaF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_D_F (src, dst, samples * 4);
-  return samples;
+  conv_D_F (conversion, src, dst, samples * 4);
 }
 
-static long
-conv_rgbF_rgb16 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbF_rgb16 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_F_16 (src, dst, samples * 3);
-  return samples;
+  conv_F_16 (conversion, src, dst, samples * 3);
 }
 
-static long
-conv_gaF_ga16 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_gaF_ga16 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_F_16 (src, dst, samples * 2);
-  return samples;
+  conv_F_16 (conversion, src, dst, samples * 2);
 }
 
 #define conv_rgbAF_rgbAD     conv_rgbaF_rgbaD
-#define conv_rgbAD_rgbAD     conv_rgbaD_rgbaF
+#define conv_rgbAD_rgbAF     conv_rgbaD_rgbaF
 #define conv_rgbAF_rgbA16    conv_rgbaF_rgba16
 #define conv_gF_g16          conv_F_16
 #define conv_gAF_gA16        conv_gaF_ga16
 
-static long
-conv_rgba8_rgbaF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgba8_rgbaF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_8_F (src, dst, samples * 4);
-  return samples;
+  conv_8_F (conversion, src, dst, samples * 4);
 }
 
-static long
-conv_rgb8_rgbF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgb8_rgbF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_8_F (src, dst, samples * 3);
-  return samples;
+  conv_8_F (conversion, src, dst, samples * 3);
 }
 
-static long
-conv_ga8_gaF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_ga8_gaF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_8_F (src, dst, samples * 2);
-  return samples;
+  conv_8_F (conversion, src, dst, samples * 2);
 }
 
 #define conv_rgbA8_rgbAF    conv_rgba8_rgbaF
 #define conv_gA8_gAF        conv_ga8_gaF
 #define conv_g8_gF          conv_8_F
 
-static long
-conv_rgba16_rgbaF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgba16_rgbaF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_16_F (src, dst, samples * 4);
-  return samples;
+  conv_16_F (conversion, src, dst, samples * 4);
 }
 
-static long
-conv_rgb16_rgbF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgb16_rgbF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_16_F (src, dst, samples * 3);
-  return samples;
+  conv_16_F (conversion, src, dst, samples * 3);
 }
 
-static long
-conv_ga16_gaF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_ga16_gaF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_16_F (src, dst, samples * 2);
-  return samples;
+  conv_16_F (conversion, src, dst, samples * 2);
 }
 
 #define conv_rgbA16_rgbAF    conv_rgba16_rgbaF
 #define conv_gA16_gAF        conv_ga16_gaF
 #define conv_g16_gF          conv_16_F
 
-static long
-conv_rgba16_rgba8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgba16_rgba8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_16_8 (src, dst, samples * 4);
-  return samples;
+  conv_16_8 (conversion, src, dst, samples * 4);
 }
 
-static long
-conv_rgb16_rgb8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgb16_rgb8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_16_8 (src, dst, samples * 3);
-  return samples;
+  conv_16_8 (conversion, src, dst, samples * 3);
 }
 
-static long
-conv_ga16_ga8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_ga16_ga8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_16_8 (src, dst, samples * 2);
-  return samples;
+  conv_16_8 (conversion, src, dst, samples * 2);
 }
 
 #define conv_rgbA16_rgbA8    conv_rgba16_rgba8
 #define conv_gA16_gA8        conv_ga16_ga8
 #define conv_g16_g8          conv_16_8
 
-static long
-conv_rgba8_rgba16 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgba8_rgba16 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_8_16 (src, dst, samples * 4);
-  return samples;
+  conv_8_16 (conversion, src, dst, samples * 4);
 }
 
-static long
-conv_rgb8_rgb16 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgb8_rgb16 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_8_16 (src, dst, samples * 3);
-  return samples;
+  conv_8_16 (conversion, src, dst, samples * 3);
 }
 
-static long
-conv_ga8_ga16 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_ga8_ga16 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
-  conv_8_16 (src, dst, samples * 2);
-  return samples;
+  conv_8_16 (conversion, src, dst, samples * 2);
 }
 
 #define conv_rgbA8_rgbA16    conv_rgba8_rgba16
@@ -357,8 +329,8 @@ conv_ga8_ga16 (unsigned char *src, unsigned char *dst, long samples)
 
 /* alpha conversions */
 
-static long
-conv_gaF_gAF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_gaF_gAF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -373,11 +345,10 @@ conv_gaF_gAF (unsigned char *src, unsigned char *dst, long samples)
       dst           += 4;
       src           += 4;
     }
-  return samples;
 }
 
-static long
-conv_gAF_gaF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_gAF_gaF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -392,13 +363,12 @@ conv_gAF_gaF (unsigned char *src, unsigned char *dst, long samples)
       dst           += 4;
       src           += 4;
     }
-  return samples;
 }
 
 /* alpha stripping and adding */
 
-static long
-conv_rgbaF_rgbF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbaF_rgbF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -415,11 +385,32 @@ conv_rgbaF_rgbF (unsigned char *src, unsigned char *dst, long samples)
       src         += 4;
       src         += 4;
     }
-  return samples;
 }
 
-static long
-conv_rgbF_rgbaF (unsigned char *src, unsigned char *dst, long samples)
+
+static void
+conv_gF_rgbaF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      *(int *) dst   = (*(int *) src);
+      dst           += 4;
+      *(int *) dst   = (*(int *) src);
+      dst           += 4;
+      *(int *) dst   = (*(int *) src);
+      dst           += 4;
+      *(float *) dst = 1.0;
+      dst           += 4;
+      src           += 4;
+    }
+}
+
+#define conv_gF_rgbAF conv_gF_rgbaF
+
+static void
+conv_rgbF_rgbaF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -437,11 +428,10 @@ conv_rgbF_rgbaF (unsigned char *src, unsigned char *dst, long samples)
       *(float *) dst = 1.0;
       dst           += 4;
     }
-  return samples;
 }
 
-static long
-conv_gaF_gF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_gaF_gF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -452,11 +442,10 @@ conv_gaF_gF (unsigned char *src, unsigned char *dst, long samples)
       src         += 4;
       src         += 4;
     }
-  return samples;
 }
 
-static long
-conv_gF_gaF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_gF_gaF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -468,7 +457,6 @@ conv_gF_gaF (unsigned char *src, unsigned char *dst, long samples)
       *(float *) dst = 1.0;
       dst           += 4;
     }
-  return samples;
 }
 
 #define conv_gF_gAF        conv_gF_gaF
@@ -477,8 +465,8 @@ conv_gF_gaF (unsigned char *src, unsigned char *dst, long samples)
 
 /* colorchannel dropping and adding */
 
-static long
-conv_gF_rgbF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_gF_rgbF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -493,11 +481,10 @@ conv_gF_rgbF (unsigned char *src, unsigned char *dst, long samples)
         }
       src += 4;
     }
-  return samples;
 }
 
-static long
-conv_gaF_rgbaF (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_gaF_rgbaF (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -515,15 +502,14 @@ conv_gaF_rgbaF (unsigned char *src, unsigned char *dst, long samples)
       dst           += 4;
       src           += 4;
     }
-  return samples;
 }
 
 #define conv_gAF_rgbAF    conv_gaF_rgbaF
 
 /* other conversions coded for some optimisation reason or sumthin */
 
-static long
-conv_rgbaF_rgbA8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbaF_rgbA8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -542,11 +528,10 @@ conv_rgbaF_rgbA8 (unsigned char *src, unsigned char *dst, long samples)
       dst++;
       src += 4;
     }
-  return samples;
 }
 
-static long
-conv_rgbaF_rgb8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbaF_rgb8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -562,11 +547,10 @@ conv_rgbaF_rgb8 (unsigned char *src, unsigned char *dst, long samples)
         }
       src += 4;
     }
-  return samples;
 }
 
-static long
-conv_rgbaF_rgb16 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbaF_rgb16 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -582,11 +566,10 @@ conv_rgbaF_rgb16 (unsigned char *src, unsigned char *dst, long samples)
         }
       src += 4;
     }
-  return samples;
 }
 
-static long
-conv_rgba8_rgbA8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgba8_rgbA8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -611,11 +594,10 @@ conv_rgba8_rgbA8 (unsigned char *src, unsigned char *dst, long samples)
       dst += 4;
       src += 4;
     }
-  return samples;
 }
 
-static long
-conv_rgbA8_rgba8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgbA8_rgba8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -641,11 +623,10 @@ conv_rgbA8_rgba8 (unsigned char *src, unsigned char *dst, long samples)
         }
       src += 4;
     }
-  return samples;
 }
 
-static long
-conv_rgb8_rgba8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgb8_rgba8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -659,13 +640,12 @@ conv_rgb8_rgba8 (unsigned char *src, unsigned char *dst, long samples)
       dst   += 4;
     }
 
-  return samples;
 }
 
 #define conv_rgb8_rgbA8    conv_rgb8_rgba8
 
-static long
-conv_rgba8_rgb8 (unsigned char *src, unsigned char *dst, long samples)
+static void
+conv_rgba8_rgb8 (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
 
@@ -677,7 +657,6 @@ conv_rgba8_rgb8 (unsigned char *src, unsigned char *dst, long samples)
       src   += 4;
       dst   += 3;
     }
-  return samples;
 }
 
 int init (void);
@@ -725,6 +704,15 @@ init (void)
     babl_component ("Ba"),
     babl_component ("A"),
     NULL);
+  const Babl *rgbAD = babl_format_new (
+    babl_model ("RaGaBaA"),
+    babl_type ("double"),
+    babl_component ("Ra"),
+    babl_component ("Ga"),
+    babl_component ("Ba"),
+    babl_component ("A"),
+    NULL);
+
   const Babl *rgbA16 = babl_format_new (
     babl_model ("RaGaBaA"),
     babl_type ("u16"),
@@ -819,6 +807,8 @@ init (void)
 
   o (rgbaF, rgbaD);
   o (rgbaD, rgbaF);
+  o (rgbAF, rgbAD);
+  o (rgbAD, rgbAF);
   o (rgbaF, rgba8);
   o (rgba8, rgbaF);
   o (rgbaF, rgba16);
@@ -840,6 +830,8 @@ init (void)
   o (gaF, ga8);
   o (gAF, gA8);
   o (gF, g8);
+  o (gF, rgbAF);
+  o (gF, rgbaF);
   o (ga8, gaF);
   o (gA8, gAF);
   o (g8, gF);
